/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.ceramics;

import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import net.minecraft.client.resources.I18n;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.*;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.Constants;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.capability.CapabilityFluidHandler;
import net.minecraftforge.fluids.capability.FluidTankPropertiesWrapper;
import net.minecraftforge.fluids.capability.IFluidHandler;
import net.minecraftforge.fluids.capability.IFluidTankProperties;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.items.ItemStackHandler;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.capability.ISmallVesselHandler;
import net.dries007.tfc.api.capability.heat.CapabilityItemHeat;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.fluids.FluidMetal;
import net.dries007.tfc.objects.fluids.FluidsTFC;
import net.dries007.tfc.util.Alloy;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.world.classic.CalenderTFC;

import net.dries007.tfc.api.capability.ISmallVesselHandler.Mode;

public class ItemSmallVessel extends ItemFiredPottery
{
    public final boolean glazed;

    public ItemSmallVessel(boolean glazed)
    {
        this.glazed = glazed;
        func_77627_a(glazed);
    }

    @Override
    @Nonnull
    public ActionResult<ItemStack> func_77659_a(World world, EntityPlayer player, @Nonnull EnumHand hand)
    {

        ItemStack stack = player.func_184586_b(hand);
        if (!world.field_72995_K && !player.func_70093_af())
        {
            IFluidHandler cap = stack.getCapability(CapabilityFluidHandler.FLUID_HANDLER_CAPABILITY, null);
            if (cap instanceof ISmallVesselHandler)
            {
                ISmallVesselHandler.Mode mode = ((ISmallVesselHandler) cap).getFluidMode();
                switch (mode)
                {
                    case INVENTORY:
                        player.openGui(TerraFirmaCraft.getInstance(), TFCGuiHandler.SMALL_VESSEL, world, 0, 0, 0);
                        break;
                    case LIQUID_MOLTEN:
                        player.openGui(TerraFirmaCraft.getInstance(), TFCGuiHandler.SMALL_VESSEL_LIQUID, world, 0, 0, 0);
                        break;
                    case LIQUID_SOLID:
                        break;
                }
            }
        }
        return new ActionResult<>(EnumActionResult.SUCCESS, stack);
    }

    @Override
    @Nonnull
    public String func_77667_c(ItemStack stack)
    {
        if (!glazed)
            return super.func_77667_c(stack);
        return super.func_77667_c(stack) + "." + EnumDyeColor.func_176766_a(stack.func_77952_i()).func_176610_l();
    }

    @Override
    public void func_150895_a(@Nonnull CreativeTabs tab, @Nonnull NonNullList<ItemStack> items)
    {
        if (!func_194125_a(tab)) return;

        if (!glazed)
            items.add(new ItemStack(this));
        else
            for (EnumDyeColor color : EnumDyeColor.values())
                items.add(new ItemStack(this, 1, color.func_176767_b()));
    }

    @Nullable
    @Override
    public ICapabilityProvider initCapabilities(ItemStack stack, @Nullable NBTTagCompound nbt)
    {
        return new SmallVesselCapability(nbt);
    }

    @Override
    public boolean canStack(@Nonnull ItemStack stack)
    {
        return false;
    }

    @Override
    public Size getSize(@Nonnull ItemStack stack)
    {
        return Size.LARGE;
    }

    @Override
    public Weight getWeight(@Nonnull ItemStack stack)
    {
        return Weight.HEAVY;
    }

    @Override
    public ItemStack getFiringResult(ItemStack input, Metal.Tier tier)
    {
        NBTTagCompound nbt = input.func_77978_p();
        // Case 1: The input is a filled vessel
        IItemHandler capItemHandler = input.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
        if (capItemHandler instanceof ISmallVesselHandler)
        {
            ISmallVesselHandler cap = (ISmallVesselHandler) capItemHandler;

            // Check if it can transform into liquid metal
            Alloy alloy = new Alloy().add(cap);
            if (alloy.isValid())
            {
                // Fill with the liquid metal
                cap.setFluidMode(true);
                cap.fill(new FluidStack(FluidsTFC.getMetalFluid(alloy.getResult()), alloy.getAmount()), true);
                cap.setTemperature(1600f);
                nbt = cap.serializeNBT();
            }

        }
        input.func_77982_d(nbt);
        return input;
    }

    // Extends ItemStackHandler for ease of use. Duplicates most of ItemHeatHandler functionality
    private class SmallVesselCapability extends ItemStackHandler implements ICapabilityProvider, ISmallVesselHandler
    {
        // todo: make cool slower.

        private final FluidTank tank;

        private float heatCapacity;
        private float meltingPoint;
        private float temperature;
        private long lastUpdateTick;

        private boolean fluidMode; // Does the stack contain molten metal?
        private IFluidTankProperties fluidTankProperties[];

        public SmallVesselCapability(@Nullable NBTTagCompound nbt)
        {
            super(4);

            tank = new FluidTank(4000);
            fluidMode = false;
            if (nbt != null)
            {
                deserializeNBT(nbt);
            }
            updateFluidData(tank.getFluid());
        }

        @Override
        public void setFluidMode(boolean fluidMode)
        {
            this.fluidMode = fluidMode;
        }

        @Override
        public Mode getFluidMode()
        {
            if (fluidMode)
            {
                return getTemperature() < meltingPoint ? Mode.LIQUID_SOLID : Mode.LIQUID_MOLTEN;
            }
            return Mode.INVENTORY;
        }

        @Override
        public float getTemperature()
        {
            return CapabilityItemHeat.adjustTemp(temperature, heatCapacity, CalenderTFC.getTotalTime() - lastUpdateTick);
        }

        @Nullable
        @Override
        public Metal getMetal()
        {
            return fluidMode && tank.getFluid() != null ? ((FluidMetal) tank.getFluid().getFluid()).getMetal() : null;
        }

        @Override
        public int getAmount()
        {
            return fluidMode ? tank.getFluidAmount() : 0;
        }

        @Override
        public boolean hasCapability(@Nonnull Capability<?> capability, @Nullable EnumFacing facing)
        {
            return capability == CapabilityFluidHandler.FLUID_HANDLER_CAPABILITY
                || capability == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY
                || capability == CapabilityItemHeat.ITEM_HEAT_CAPABILITY;
        }

        @Nullable
        @Override
        @SuppressWarnings("unchecked")
        public <T> T getCapability(@Nonnull Capability<T> capability, @Nullable EnumFacing facing)
        {
            return hasCapability(capability, facing) ? (T) this : null;
        }

        @Override
        public NBTTagCompound serializeNBT()
        {
            NBTTagCompound nbt = new NBTTagCompound();
            fluidMode = tank.getFluidAmount() > 0;
            nbt.func_74757_a("fluidMode", fluidMode);

            nbt.func_74776_a("heat", getTemperature());
            nbt.func_74772_a("ticks", CalenderTFC.getTotalTime());

            if (fluidMode)
            {
                // Save fluid data
                NBTTagCompound fluidData = new NBTTagCompound();
                tank.writeToNBT(fluidData);
                nbt.func_74782_a("fluids", fluidData);
            }
            else
            {
                // Save item data
                nbt.func_74782_a("items", super.serializeNBT());
            }
            return nbt;
        }

        @Override
        public void deserializeNBT(NBTTagCompound nbt)
        {
            temperature = nbt.func_74760_g("heat");
            lastUpdateTick = nbt.func_74763_f("ticks");
            fluidMode = nbt.func_74767_n("fluidMode");

            if (fluidMode && nbt.func_150297_b("fluids", Constants.NBT.TAG_COMPOUND))
            {
                // Read fluid contents
                tank.readFromNBT(nbt.func_74775_l("fluids"));
            }
            else if (!fluidMode && nbt.func_150297_b("items", Constants.NBT.TAG_COMPOUND))
            {
                // Read item contents
                super.deserializeNBT(nbt.func_74775_l("items"));
            }

        }

        @Override
        public IFluidTankProperties[] getTankProperties()
        {
            if (fluidTankProperties == null)
            {
                fluidTankProperties = new IFluidTankProperties[] {new FluidTankPropertiesWrapper(tank)};
            }
            return fluidTankProperties;
        }

        @Override
        public int fill(FluidStack resource, boolean doFill)
        {
            if (resource.getFluid() instanceof FluidMetal)
            {
                updateFluidData(resource);
                return tank.fill(resource, doFill);
            }
            return 0;
        }

        @Nullable
        @Override
        public FluidStack drain(FluidStack resource, boolean doDrain)
        {
            if (getFluidMode() == Mode.LIQUID_MOLTEN)
                return tank.drain(resource, doDrain);
            return null;
        }

        @Nullable
        @Override
        public FluidStack drain(int maxDrain, boolean doDrain)
        {
            if (getFluidMode() == Mode.LIQUID_MOLTEN)
                return tank.drain(maxDrain, doDrain);
            return null;
        }

        @Override
        public void setTemperature(float temperature)
        {
            this.temperature = temperature;
            this.lastUpdateTick = CalenderTFC.getTotalTime();
        }

        @Override
        public boolean isMolten()
        {
            TerraFirmaCraft.getLog().debug("Check Molten: {} >= {}", getTemperature(), meltingPoint);
            return getTemperature() >= meltingPoint;
        }

        @SideOnly(Side.CLIENT)
        @Override
        public void addHeatInfo(ItemStack stack, List<String> text, boolean clearStackNBT)
        {
            Metal metal = getMetal();
            if (metal != null)
            {
                String desc = TextFormatting.DARK_GREEN + I18n.func_135052_a(Helpers.getTypeName(metal)) + ": " + I18n.func_135052_a("tfc.tooltip.units", getAmount());
                if (isMolten())
                    desc += " - " + I18n.func_135052_a("tfc.tooltip.liquid");
                text.add(desc);
            }
            ISmallVesselHandler.super.addHeatInfo(stack, text, false); // Never clear the NBT based on heat alone
        }

        private void updateFluidData(@Nullable FluidStack fluid)
        {
            if (fluid != null && fluid.getFluid() instanceof FluidMetal)
            {
                Metal metal = ((FluidMetal) fluid.getFluid()).getMetal();
                this.meltingPoint = metal.getMeltTemp();
                this.heatCapacity = metal.getSpecificHeat();
            }
            else
            {
                this.meltingPoint = 1000;
                this.heatCapacity = 1;
            }

        }
    }

}
