/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nullable;

import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.inventory.ItemStackHelper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFlintAndSteel;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.items.ItemsTFC;
import net.dries007.tfc.objects.items.wood.ItemLogTFC;
import net.dries007.tfc.util.IFireable;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

@MethodsReturnNonnullByDefault
public class TEPitKiln extends TileEntity implements ITickable
{
    public static final ResourceLocation ID = new ResourceLocation(MOD_ID, "pit_kiln");

    public static final int STRAW_NEEDED = 8;
    public static final int WOOD_NEEDED = 8;
    private final NonNullList<ItemStack> logs = NonNullList.func_191197_a(WOOD_NEEDED, ItemStack.field_190927_a);
    private final NonNullList<ItemStack> straw = NonNullList.func_191197_a(STRAW_NEEDED, ItemStack.field_190927_a);
    private final NonNullList<ItemStack> items = NonNullList.func_191197_a(4, ItemStack.field_190927_a);
    private int burnTicksToGo;

    @Override
    public void func_73660_a()
    {
        if (burnTicksToGo > 0)
        {
            burnTicksToGo--;
            BlockPos above = func_174877_v().func_177982_a(0, 1, 0);
            if (field_145850_b.func_175623_d(above))
            {
                field_145850_b.func_175656_a(above, Blocks.field_150480_ab.func_176223_P());
            }
            else
            {
                IBlockState stateAbove = field_145850_b.func_180495_p(above);
                if (stateAbove.func_185904_a() != Material.field_151581_o)
                {
                    // todo: decide what to do now.
                    burnTicksToGo = 0;
                    return;
                }
            }
            if (burnTicksToGo == 0)
            {
                straw.clear();
                logs.clear();
                for (int i = 0; i < items.size(); i++)
                {
                    ItemStack stack = items.get(i);
                    Item item = stack.func_77973_b();
                    if (!(item instanceof IFireable)) continue;
                    IFireable fireable = ((IFireable) item);
                    items.set(i, fireable.getFiringResult(stack, Metal.Tier.TIER_I));
                }
                field_145850_b.func_175698_g(above);
                updateBlock();
            }
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound compound)
    {
        super.func_145839_a(compound);
        burnTicksToGo = compound.func_74762_e("burnTicksToGo");
        ItemStackHelper.func_191283_b(compound.func_74775_l("items"), items);
        ItemStackHelper.func_191283_b(compound.func_74775_l("straw"), straw);
        ItemStackHelper.func_191283_b(compound.func_74775_l("logs"), logs);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound)
    {
        super.func_189515_b(compound);
        compound.func_74772_a("burnTicksToGo", burnTicksToGo);
        compound.func_74782_a("items", ItemStackHelper.func_191282_a(new NBTTagCompound(), items));
        compound.func_74782_a("straw", ItemStackHelper.func_191282_a(new NBTTagCompound(), straw));
        compound.func_74782_a("logs", ItemStackHelper.func_191282_a(new NBTTagCompound(), logs));
        return compound;
    }

    @Nullable
    @Override
    public SPacketUpdateTileEntity func_189518_D_()
    {
        return new SPacketUpdateTileEntity(field_174879_c, 127, func_189517_E_());
    }

    @Override
    public NBTTagCompound func_189517_E_()
    {
        return func_189515_b(new NBTTagCompound());
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt)
    {
        func_145839_a(pkt.func_148857_g());
        updateBlock();
    }

    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox()
    {
        return new AxisAlignedBB(func_174877_v(), func_174877_v().func_177982_a(1, 1, 1));
    }

    public boolean isLit()
    {
        return burnTicksToGo > 0;
    }

    public boolean hasFuel()
    {
        return !(logs.stream().anyMatch(ItemStack::func_190926_b) || straw.stream().anyMatch(ItemStack::func_190926_b));
    }

    @SuppressWarnings("ConstantConditions")
    public void onRightClick(EntityPlayer player, ItemStack item, boolean x, boolean z)
    {
        if (isLit()) return;
        int count = getStrawCount();
        int slot = 0;
        if (x) slot += 1;
        if (z) slot += 2;
        if (item.func_190926_b())
        {
            if (getLogCount() > 0)
            {
                ItemStack itemStack = logs.stream().filter(i -> !i.func_190926_b()).findFirst().get();
                player.func_191521_c(itemStack.func_77979_a(1));
                updateBlock();
                return;
            }
            if (getStrawCount() > 0)
            {
                ItemStack itemStack = straw.stream().filter(i -> !i.func_190926_b()).findFirst().get();
                player.func_191521_c(itemStack.func_77979_a(1));
                updateBlock();
                return;
            }
            ItemStack current = items.get(slot);
            if (current.func_190926_b()) return;
            player.func_191521_c(current.func_77979_a(1));
            items.set(slot, ItemStack.field_190927_a);
            updateBlock();
            if (items.stream().filter(ItemStack::func_190926_b).count() == 4)
            {
                field_145850_b.func_175698_g(field_174879_c);
                return;
            }
            return;
        }
        if (IFireable.fromItem(item.func_77973_b()) != null)
        {
            ItemStack current = items.get(slot);
            if (!current.func_190926_b()) return;
            items.set(slot, item.func_77979_a(1));
            updateBlock();
            return;
        }
        if (item.func_77973_b() == ItemsTFC.HAY && count < STRAW_NEEDED)
        {
            addStraw(item.func_77979_a(1));
            updateBlock();
            return;
        }
        if (item.func_77973_b() == Item.func_150898_a(BlocksTFC.THATCH) && count <= STRAW_NEEDED - 4)
        {
            item.func_190918_g(1);
            addStraw(new ItemStack(ItemsTFC.HAY));
            addStraw(new ItemStack(ItemsTFC.HAY));
            addStraw(new ItemStack(ItemsTFC.HAY));
            addStraw(new ItemStack(ItemsTFC.HAY));
            updateBlock();
            return;
        }
        if (count < STRAW_NEEDED) return;
        count = getLogCount();
        if (item.func_77973_b() instanceof ItemLogTFC && count < WOOD_NEEDED)
        {
            addLog(item.func_77979_a(1));
            updateBlock();
            return;
        }
        if (count < WOOD_NEEDED) return;
        if (item.func_77973_b() instanceof ItemFlintAndSteel)
        {
            tryLight();
        }
    }

    public void updateBlock()
    {
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        field_145850_b.func_184138_a(field_174879_c, state, state, 3); // sync TE
        func_70296_d(); // make sure everything saves to disk
    }

    public void onBreakBlock()
    {
        items.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(), i));
        straw.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(), i));
        logs.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(), i));
    }

    public NonNullList<ItemStack> getItems()
    {
        return items;
    }

    public int getLogCount()
    {
        return (int) logs.stream().filter(i -> !i.func_190926_b()).count();
    }

    public int getStrawCount()
    {
        return (int) straw.stream().filter(i -> !i.func_190926_b()).count();
    }

    public boolean tryLight()
    {
        if (!hasFuel()) return false;
        BlockPos above = field_174879_c.func_177982_a(0, 1, 0);
        if (!Blocks.field_150480_ab.func_176196_c(field_145850_b, above)) return false;
        for (EnumFacing facing : EnumFacing.Plane.HORIZONTAL)
        {
            if (!field_145850_b.isSideSolid(field_174879_c.func_177972_a(facing), facing.func_176734_d())) return false;
        }
        burnTicksToGo = ConfigTFC.GENERAL.pitKilnTime;
        updateBlock();
        field_145850_b.func_175656_a(above, Blocks.field_150480_ab.func_176223_P());
        return true;
    }

    private void addStraw(ItemStack stack)
    {
        for (int i = 0; i < straw.size(); i++)
        {
            if (!straw.get(i).func_190926_b()) continue;
            straw.set(i, stack);
            return;
        }
    }

    private void addLog(ItemStack stack)
    {
        for (int i = 0; i < logs.size(); i++)
        {
            if (!logs.get(i).func_190926_b()) continue;
            logs.set(i, stack);
            return;
        }
    }
}
